//--
//--	dogwind.cpp
//--
//--	TDogWindow Class Module
//--


#include "samppch.h"
#include <stdlib.h>
#include <time.h>
#include <owl\window.h>
#include <owl\module.h>
#include <owl\point.h>
#include <owl\dc.h>
#include <owl\color.h>
#include <owl\gdiobjec.h>
#include "sampdefs.h"
#include "sample.rh"
#include "wingdc.h"
#include "animwind.h"
#include "dogwind.h"


DEFINE_RESPONSE_TABLE1(TDogWindow, TBitmapAnimatorWindow)
	EV_WM_LBUTTONDOWN,
	EV_WM_SIZE,
END_RESPONSE_TABLE;


//----------------------------
//  TDogWindow::TDogWindow()
//----------------------------
TDogWindow::TDogWindow(TWindow *pParWin,
								  const char far *pszTitle, TModule *pMod) :
					 TBitmapAnimatorWindow(pParWin,pszTitle,pMod)
{
	pBkgndDC = 0;  // Allocated in SetupWindow()
	pBkgndBitmap = 0;  // Allocated in SetupWindow()
	pSpriteDIB = new TDib("DOGGIE.BMP");
	SpriteRect.SetNull();
}


//-----------------------------
//  TDogWindow::~TDogWindow()
//-----------------------------
TDogWindow::~TDogWindow()
{
	Destroy();

	delete pSpriteDIB;
	delete pBkgndDC;  // Usually deleted in CleanupWindow()
	delete pBkgndBitmap;  // Usually deleted in CleanupWindow()
}


//-----------------------------
//  TDogWindow::SetupWindow()
//-----------------------------
void TDogWindow::SetupWindow()
{
	TBitmapAnimatorWindow::SetupWindow();

	// Load background DIB as a bitmap and select it into a memory DC for
	//   fast blts to the offscreen buffer (WinGDC)
	if (pBufferDC)
	{
		pBkgndBitmap = new TBitmap(*pBufferDC,TDib("DIBBK.BMP"));
		pBkgndDC = new TMemoryDC(*pBufferDC);
		pBkgndDC->SelectObject(*pPalette);
		pBkgndDC->SelectObject(*pBkgndBitmap);
		BkgndRect.SetNull();
	}
}


//-------------------------------
//  TDogWindow::CleanupWindow()
//-------------------------------
void TDogWindow::CleanupWindow()
{
	delete pBkgndDC;
	pBkgndDC = 0;
	delete pBkgndBitmap;
	pBkgndBitmap = 0;

	TBitmapAnimatorWindow::CleanupWindow();
}


//-----------------------------
//  TDogWindow::InitPalette()
//-----------------------------
void TDogWindow::InitPalette()
{
	// Allocate and init a temporary PALETTEENTRY array
	uint nNumEntries = pPalette->GetNumEntries();
	PALETTEENTRY *pPalEntries = new PALETTEENTRY[nNumEntries];

	// fill the array with the palette entries
	pPalette->GetPaletteEntries(0,nNumEntries,pPalEntries);

	// copy palette entries from pSpriteDIBs color table
	TRgbQuad *pDIBColors = pSpriteDIB->GetColors();

	for (uint nCounter = 10; nCounter < 246; ++nCounter)
	{
		pPalEntries[nCounter].peRed = pDIBColors[nCounter].rgbRed;
		pPalEntries[nCounter].peGreen = pDIBColors[nCounter].rgbGreen;
		pPalEntries[nCounter].peBlue = pDIBColors[nCounter].rgbBlue;
	}

	// place the entries back in the palette
	pPalette->SetPaletteEntries(0,nNumEntries,pPalEntries);

	// Free the temporary PALETTEENTRY array
	delete pPalEntries;
}


//------------------------------------
//  TDogWindow::DrawForeground()
//------------------------------------
void TDogWindow::DrawForeground(TRect &)
{
	// Draw the doggie sprite in offscreen buffer
	//    (Should we check if SpriteRect is in DirtyRect???)
	TransparentDIBits(SpriteRect.left,SpriteRect.top,pSpriteDIB,243);
}


//------------------------------------
//  TDogWindow::DrawBackground()
//------------------------------------
void TDogWindow::DrawBackground(TRect &UpdateRect)
{
	// Clear the UpdateRect portion of the offscreen buffer to black
	TBitmapAnimatorWindow::DrawBackground(UpdateRect);

	// Blt background bitmap to offscreen buffer
	TRect BltRect = UpdateRect & BkgndRect;
	if (!BltRect.IsEmpty())
	{
		pBufferDC->BitBlt(BltRect,
						  *pBkgndDC,
						  TPoint(BltRect.left - BkgndRect.left,
										 BltRect.top - BkgndRect.top),
						  SRCCOPY);
	}
}


//--------------------------------------
//  TDogWindow::EvLButtonDown()
//--------------------------------------
void TDogWindow::EvLButtonDown(uint,TPoint &)
{
	TPoint BaseCursorPoint, CursorPoint;

	::GetCursorPos(&BaseCursorPoint);
	ScreenToClient(BaseCursorPoint);

	// If mouse was clicked on doggie sprite then let the user "drag" it
	if (::PtInRect(&SpriteRect,BaseCursorPoint))
	{
		MSG  LoopMsg;
		int nOffsetX = SpriteRect.left - BaseCursorPoint.x;
		int nOffsetY = SpriteRect.top - BaseCursorPoint.y;

		LoopMsg.message = 0;

		SetCapture();

		while (LoopMsg.message != WM_LBUTTONUP &&
							   (LoopMsg.message != WM_MOUSEMOVE ||
										   (LoopMsg.wParam & MK_LBUTTON)))
		{
			if (::PeekMessage(&LoopMsg,0,0,0,PM_REMOVE))
			{
				::TranslateMessage(&LoopMsg);
				::DispatchMessage(&LoopMsg);
			}

			// If mouse cursor moved since the last time around then
			//   draw the sprite at the new position
			::GetCursorPos(&CursorPoint);
			ScreenToClient(CursorPoint);
			if (CursorPoint != BaseCursorPoint)
			{
				// Calc DirtyRect and new doggie SpriteRect
				DirtyRect = SpriteRect;
				SpriteRect.left = CursorPoint.x + nOffsetX;
				SpriteRect.top = CursorPoint.y + nOffsetY;
				SpriteRect.right = SpriteRect.left + pSpriteDIB->Width();
				SpriteRect.bottom = SpriteRect.top + pSpriteDIB->Height();
				DirtyRect |= SpriteRect;

				// Update offscreen buffer and flush it to the screen
				RefreshScreen();

				// Update BaseCursorPoint so we know if the mouse moved
				BaseCursorPoint = CursorPoint;
			}
		}

		ReleaseCapture();

		GetApplication()->ResumeThrow();
	}

	DefaultProcessing();
}


//------------------------
//  TDogWindow::EvSize()
//------------------------
void TDogWindow::EvSize(uint nType, TSize &Size)
{
	if (pBkgndBitmap && pSpriteDIB && Size.cx && Size.cy)
	{
		// Center the background bitmap
		BkgndRect.left = (Size.cx - pBkgndBitmap->Width()) / 2;
		BkgndRect.top = (Size.cy - pBkgndBitmap->Height()) / 2;
		BkgndRect.right = BkgndRect.left + pBkgndBitmap->Width();
		BkgndRect.bottom = BkgndRect.top + pBkgndBitmap->Height();

		// Center the sprite bitmap
		SpriteRect.left = (Size.cx - pSpriteDIB->Width()) / 2;
		SpriteRect.top = (Size.cy - pSpriteDIB->Height()) / 2;
		SpriteRect.right = SpriteRect.left + pSpriteDIB->Width();
		SpriteRect.bottom = SpriteRect.top + pSpriteDIB->Height();
	}

	TBitmapAnimatorWindow::EvSize(nType,Size);
}


